#include <iostream>
#include <vector>
#include <algorithm>
#include <set>
#include <queue>

#define upmin(a, b) a = min(a, b)
#define upmax(a, b) a = max(a, b)
#define pr(x) cout << x << endl
#define wpr(x) cout << #x << ": " << x << endl
#define wprv(x) cout << #x << ": " << endl; for(auto it : x) cout << it << " "; cout << endl;
#define wprvv(x) cout << #x << ": " << endl; for(auto it : x) {for(auto it2 : it) cout << it2<< " "; cout << endl;}
#define rep(i, s, e) for(ll i = s; i < e; i++)

using namespace std;
using ll = long long;
using vll = vector<ll>;
using vll = vector<ll>;
using vvll = vector<vll>;
using pll = pair<ll, ll>;
using vpll = vector<pll>;
using vvpll = vector<vpll>;
using vb = vector<bool>;


/*

I think the solution might be as follows:

Find for each one the first time he knows the news
Then, for query, find the number of nodes that know the news before the current time.
This can be done by using euler tour to flatten the tree to an array, and then make a persistent segtree.

To avoid the problem that the queries are only to k sons, we calculate the queries in
DECREASING END order.
And we update the nodes only when we got to them.
So the limit on k is removed.
Let's first make an euler tour.

Let's calculate got naively at first.

To calc got not naively we should make a segtree like before, sort the queries in
INCREASING END order,
And then when we finish working on a node we take his answer.
This is a lazy segtree, but simple one (with only upmin())

Good, now let's make solve_queries() faster.
To do this we make a persistent segtree.

Maybe calc for each node, to which ranges he adds.

Let's solve k = n
in this case, one segtree is enough
*/
const ll inf = 1e18;
struct Seg {
	vll a;
	ll asz;
	void make(ll n) {
		for (asz = 1; asz < n; asz <<= 1);
		a.resize(asz * 2, 0);
	}
	void up(ll i, ll x) {
		a[i += asz] = x;
		for (i >>= 1; i; i >>= 1) a[i] = a[2 * i] + a[2 * i + 1];
	}
	ll q(ll l, ll r) {
		l += asz, r += asz;
		ll res = 0;
		while (l <= r) {
			if (l % 2 == 1) res += a[l++];
			if (r % 2 == 0) res += a[r--];
			l >>= 1, r >>= 1;
		}
		return res;
	}
};

struct query {
	ll x, k, t;
	ll type; // 1 or 2
	query(ll x, ll k, ll t, ll type) : x(x), k(k), t(t), type(type) {}

	void print()
	{
		cout << "{ x: " << x << ", k: " << k << ", t: " << t << "}" << endl;
	}
};

ll n, Q;
vvll tree;
vll par;
vb know;
vll got; // first time the got the news
vector<query> q1;
vector<query> q2;
vll tour, sz, level, tin;
ll timer;
vll nodes; // sorted vy level


void euler_tour(ll cur)
{
	tin[cur] = timer++;
	tour.push_back(cur);
	for (auto& nei : tree[cur]) {
		if (nei == par[cur]) continue;
		euler_tour(nei);
	}
}

ll calc_sz_level(ll cur) {
	if (par[cur] == -1) level[cur] = 0;
	else level[cur] = level[par[cur]] + 1;
	ll csz = 1;
	for (auto& nei : tree[cur]) {
		if (nei == par[cur]) continue;
		csz += calc_sz_level(nei);
	}
	sz[cur] = csz;
	return sz[cur];
}
void spread(ll cur, ll x, ll t)
{
	if (x < 0) return;
	upmin(got[cur], t);
	for (auto& nei : tree[cur]) {
		if (nei == par[cur]) continue;
		spread(nei, x - 1, t);
	}
}

void calc_got()
{
	for (auto& q : q1) {
		spread(q.x, q.k, q.t);
	}
}

bool comp_dec_end(const query& qu1, const query& qu2)
{
	return level[qu1.x] + qu1.k < level[qu2.x] + qu2.k;
}

void solve_queries()
{
	// sort the queries first
	sort(q2.begin(), q2.end(), comp_dec_end);
	/*
	for (auto q : q2) {
		q.print();
	}
	*/
	/*
	sort the nodes in increasing level - this is nodes
	*/
	vll now(n, inf); // who knows the news for now
	ll p = 0; // pointer to nodes
	vll ans(Q+1, -1);
	Seg seg;
	seg.make(n);
	for (auto& q : q2) {
		while (p < n && level[nodes[p]] <= level[q.x] + q.k) {
			//now[nodes[p]] = got[nodes[p]];
			if(got[nodes[p]] == 0) seg.up(tin[nodes[p]], 1);
			p++;
		}
		ll cures = 0;
		//wpr(q.x);
		//cout << "q.x: " << q.x + 1 << endl;
		/*
		for (ll i = 0; i < sz[q.x]; i++) {
			ll c = tour[tin[q.x] + i];
			//wpr(c);
			//cout << "c: " << c+1 << endl;

			if (now[c] < q.t) cures++;
		}
		*/
		cures = seg.q(tin[q.x], tin[q.x] + sz[q.x] - 1);
		ans[q.t] = cures;
	}
	//wprv(ans);
	for (auto v : ans) {
		if (v == -1) continue;
		pr(v);
	}
}



bool comp_level(ll a, ll b) {
	return level[a] < level[b];
}

void make_nodes()
{
	nodes.resize(n);
	rep(i, 0, n) nodes[i] = i;
	sort(nodes.begin(), nodes.end(), comp_level);
}
void solve()
{
	cin >> n;
	tree.resize(n);
	know.resize(n, 0);
	par.resize(n, -1);
	rep(i, 0, n - (ll)1) {
		ll pap, son;
		cin >> pap >> son;
		pap--, son--;
		par[son] = pap;
		tree[pap].push_back(son);
	}
	got.resize(n, inf);
	rep(i, 0, n) {
		ll b; cin >> b;
		know[i] = b;
		if (know[i]) got[i] = 0;
	}

	cin >> Q;
	ll curtime = 0;
	
	rep(asdf, 0, Q) {
		curtime++;
		ll opt, x, k;
		cin >> opt >> x >> k;
		x--;

		if (opt == 1) {
			q1.push_back(query(x, k, curtime, (ll)1));
		}
		else {
			q2.push_back(query(x, k, curtime, (ll)2));
		}
	}
	
	tin.resize(n);
	timer = 0;
	euler_tour(0);
	//wprv(tin);
	//wprv(tour);
	sz.resize(n);
	level.resize(n);
	calc_sz_level(0);
	//wprv(level);
	//calc_got();
	//wprv(got);
	make_nodes();
	//wprv(nodes);
	solve_queries();
}


int main()
{
	ios_base::sync_with_stdio(false);
	cin.tie(0);
	solve();
}

/*


10
1 2
1 3
3 4
3 5
3 6
4 7
4 8
8 9
8 10
0 1 0 1 0
1 0 1 0 1
9
2 1 1
2 4 4
2 3 0
1 1 2
2 3 4
1 4 1
2 1 1
2 4 4
2 3 2



1
3
0
6
3
4
6

2
2 1
0 0
0

*/