#include <iostream>
#include <vector>
#include <algorithm>
#include <cassert>
#include <cmath>

using ll = long long;

int const nmax = 200000;
std::vector<int> g[5 +nmax];
int level[5 + nmax], start[1 + nmax], stop[1 + nmax], ptr = 0;
int rev[5 + nmax];

void dfs(int node ){
  start[node] = ++ptr;
  rev[ptr] = node;
  
  for(int h = 0; h < g[node].size(); h++) {
    int to = g[node][h];
    level[to] = level[node] + 1;
    dfs(to);
  }
  stop[node] = ptr;
}

class SegmentTree{
private:
  std::vector<std::pair<int,int>> aint;
public:
  SegmentTree(int n) {
    aint.resize(1 + 4 * n);
  }
  void update(int node, int from, int to, int x, int val){
    if(from < to) {
      int mid = (from + to) / 2;
      if(x <= mid)
        update(node * 2, from, mid, x, val);
      else
        update(node * 2 + 1, mid + 1, to, x, val);
      aint[node] = std::min(aint[node * 2], aint[node * 2 + 1]);
    } else
      aint[node] = {val, x};
  }

  std::pair<int,int> query(int node, int from, int to, int x, int y) {
    if(from == x && to == y)
      return aint[node];
    else {
      int mid = (from + to) / 2;
      if(x <= mid && y <= mid)
        return query(node * 2, from, mid, x, y);
      else if(mid + 1 <= x && mid + 1 <= y)
        return query(node * 2 + 1, mid + 1, to, x, y);
      else
        return std::min(query(node * 2, from, mid, x, mid),
                        query(node * 2 + 1, mid + 1, to, mid + 1, y));
    }
  }
};

class FenwickTree {
private:
  std::vector<int> aib;
  int n;
public:
  FenwickTree(int n_ = 0) {
    n = n_;
    aib.resize(1 + n);
  }
  void update(int pos, int val) {
    for(int x = pos; x <= n; x += (x ^ (x & (x - 1))))
      aib[x] += val;
  }
  int query(int pos) {
    int result = 0;
    if(n < pos)
      pos = n;
    for(int x = pos; 0 < x; x = (x & (x - 1)))
      result += aib[x];
    return result;
  }
};

class SegmentTree2{
private:
  std::vector<FenwickTree> aint;
  std::vector<int> startbound, stopbound;
public:
  SegmentTree2(int n) {
    aint.resize(1 + 4 * n);
    startbound.resize(1 + 4 * n);
    stopbound.resize(1 + 4 * n);
  }

  void build(int node, int from, int to) {
    startbound[node] = stopbound[node] = level[rev[from]];
    if(from < to) {
      int mid = (from + to) / 2;
      build(node * 2, from, mid);
      build(node * 2 + 1, mid + 1, to);
      startbound[node] = std::min(startbound[node], std::min(startbound[node * 2], startbound[node * 2 + 1]));
      stopbound[node] = std::max(stopbound[node], std::max(stopbound[node * 2], stopbound[node * 2 + 1]));
    }
    aint[node] = FenwickTree(stopbound[node] - startbound[node] + 1);
  }

  void update(int node, int from, int to, int x, int pos) {
    aint[node].update(pos - startbound[node] + 1, 1);
    if(from < to) {
      int mid = (from + to) / 2;
      if(x <= mid)
        update(node * 2, from, mid, x, pos);
      else
        update(node * 2 + 1, mid + 1, to, x, pos);
    }
  }
  int query(int node, int from, int to, int x, int y, int pos, int pos2) {
    if(from == x && to == y)
      return aint[node].query(pos2 - startbound[node] + 1) - aint[node].query(pos - startbound[node]);
    else {
      int mid = (from + to) / 2;
      if(x <= mid && y <= mid)
        return query(node * 2, from, mid, x, y, pos, pos2);
      else if(mid + 1 <= x && mid + 1 <= y)
        return query(node * 2 + 1, mid + 1, to, x, y, pos, pos2);
      else
        return query(node * 2, from, mid, x, mid, pos, pos2) + 
               query(node * 2 + 1, mid + 1, to, mid + 1, y, pos, pos2);
    }
  }
};


int main() {
  std::ios::sync_with_stdio(0);
  std::cin.tie(0);

  int n;
  std::cin >> n;
  for(int i = 1;i < n; i++) {
    int x, y;
    std::cin >> x >> y;
    g[x].push_back(y);
  }
  dfs(1);
  SegmentTree aint(n);
  for(int i = 1;i <= n; i++)
    aint.update(1, 1, n, start[i], level[i]);
  
  SegmentTree2 aint2(n);
  aint2.build(1, 1, n);

  for(int i = 1; i <= n; i++) {
    int op;
    std::cin >> op;
    if(op == 1) {
      aint.update(1, 1, n, start[i], nmax + 1);
      aint2.update(1, 1, n, start[i], level[i]);
    }
  }
  int q;
  std::cin >> q;
  for(int i = 1; i <= q; i++) {
    int op, x, k;
    std::cin >> op >> x >> k;
    if(op == 1) {
      std::pair<int,int> sol = aint.query(1, 1, n, start[x], stop[x]);
      while(sol.first <= level[x] + k && sol.first <= nmax) {
        int node = rev[sol.second];
        aint.update(1, 1, n, start[node], nmax + 1);
        aint2.update(1, 1, n, start[node], level[node]);
        sol = aint.query(1, 1, n, start[x], stop[x]);
      }
    } else {
      std::cout << aint2.query(1, 1, n, start[x], stop[x], level[x], level[x] + k) << '\n'; 
    }
  }
  return 0;
}
