#include <iostream>
#include <vector>

using namespace std;

const int max_s = 2e5 + 1; 
int val, tm;  
int b[max_s]; 
int rnk[max_s];  
int dis[max_s];
int fin[max_s]; 

const int size = (1 << 18); 
int segTree[2*size]; 
bool down[2*size]; 

void init(int n){
    for (int i = 1; i <= n; ++i){
        segTree[size + dis[i]] = b[i]; 
    }
    for (int i = size - 1; i > 0; --i) segTree[i] = segTree[2*i] + segTree[2*i+1]; 
}

void push(int root, int left, int right){
    if (down[root]){
        segTree[root] = right - left + 1;
        if (root < size){
            down[root*2] = down[root*2+1] = true; 
        }
    }
}

void pull(int root){
    if (root < size) segTree[root] = segTree[root*2] + segTree[root*2+1]; 
}

void update(int first, int last, int root = 1, int left = 0, int right = size - 1){
    if (first > right || last < left) return; 
    if (first <= left && last >= right){
        segTree[root] = (right - left + 1);
        down[root] = true; 
        return; 
    }
    int mid = (left + right) >> 1; 
    push(root, left, right);
    update(first, last, root*2, left, mid);
    update(first, last, root*2+1, mid+1, right); 
    pull(root); 
}

int query(int first, int last, int root = 1, int left = 0, int right = size - 1){
    if (first > right || last < left) return 0; 
    if (first <= left && last >= right){
        return segTree[root]; 
    }
    int mid = (left + right) >> 1; 
    push(root, left, right);
    int d = query(first, last, root*2, left, mid) + 
            query(first, last, root*2+1, mid+1, right); 
    pull(root);    
    return d; 
}

void dfs(int root, vector<vector<int>> & adj){
    dis[root] = tm++; 
    for (int c : adj[root]){
        rnk[c] = rnk[root] + 1; 
        dfs(c, adj); 
    }
    fin[root] = tm - 1; 
}

int main(){
    ios_base::sync_with_stdio(false); 
    cin.tie(0);
    int n; cin >> n; 
    vector<vector<int>> adj(n + 1, vector<int>()); 
    
    for (int i = 1; i < n; ++i){
        int x, y; cin >> x >> y; 
        adj[x].push_back(y); 
    }
    
    for (int i = 1; i <= n; ++i) cin >> b[i]; 
    dfs(1, adj); 

    init(n); 
    int q; cin >> q; 

    while (q--){
        int m, x, k; cin >> m >> x >> k; 
        if (m == 1) update(dis[x], fin[x]); 
        else cout << query(dis[x], fin[x]) << "\n"; 
    }
}
