#include <iostream>
#include <vector>
#include <algorithm>

using namespace std;
using pi = pair<int, int>; 
using ppi = pair<pi, pi>; 

const int max_s = 2e5 + 1; 
int val, tm;  
int b[max_s]; 
int rnk[max_s];  
int dis[max_s];
int fin[max_s]; 

const int size = (1 << 18); 
int segTree[2*size]; 
bool down[2*size]; 

void init(int n){
    for (int i = 1; i <= n; ++i){
        segTree[size + dis[i]] = b[i]; 
    }
    for (int i = size - 1; i > 0; --i) segTree[i] = segTree[2*i] + segTree[2*i+1]; 
}

void push(int root, int left, int right){
    if (down[root]){
        segTree[root] = right - left + 1;
        if (root < size){
            down[root*2] = down[root*2+1] = true; 
            int mid = (left + right) >> 1; 
            segTree[root*2] = mid - left + 1; 
            segTree[root*2+1] = right - mid; 
        }
    }
}

void pull(int root){
    if (root < size) segTree[root] = segTree[root*2] + segTree[root*2+1]; 
}

void update(int first, int last, int root = 1, int left = 0, int right = size - 1){
    if (first > right || last < left) return; 
    if (first <= left && last >= right){
        segTree[root] = (right - left + 1);
        down[root] = true; 
        return; 
    }
    int mid = (left + right) >> 1; 
    push(root, left, right);
    update(first, last, root*2, left, mid);
    update(first, last, root*2+1, mid+1, right); 
    pull(root); 
}

int query(int first, int last, int root = 1, int left = 0, int right = size - 1){
    push(root, left, right); 
    if (first > right || last < left) return 0; 
    if (first <= left && last >= right){
        return segTree[root]; 
    }
    int mid = (left + right) >> 1; 
    int d = query(first, last, root*2, left, mid) + 
            query(first, last, root*2+1, mid+1, right); 
    pull(root);    
    return d; 
}

void dfs(int root, vector<vector<int>> & adj){
    dis[root] = tm++; 
    for (int c : adj[root]){
        rnk[c] = rnk[root] + 1; 
        dfs(c, adj); 
    }
    fin[root] = tm - 1; 
}

int main(){
    ios_base::sync_with_stdio(false); 
    cin.tie(0);
    int n; cin >> n; 
    vector<vector<int>> adj(n + 1, vector<int>()); 
    
    for (int i = 1; i < n; ++i){
        int x, y; cin >> x >> y; 
        adj[x].push_back(y); 
    }
    
    for (int i = 1; i <= n; ++i) cin >> b[i]; 
    dfs(1, adj); 

    int q; cin >> q; 
    vector<ppi> queries;
    for (int i = 0; i < q; ++i){
        int m, x, k; cin >> m >> x >> k; 
        queries.push_back(ppi(pi(rnk[x]+k, i), pi(x, k))); 
    }
    vector<pi> updates;
    for (int i = 1; i <= n; ++i){
        updates.push_back(pi(rnk[i], i)); 
    }

    sort(queries.begin(), queries.end()); 
    sort(updates.begin(), updates.end()); 

    vector<int> res(q); 

    int ind_up = 0, ind_q = 0; 
    while (ind_q < q){
        while (ind_up < n && updates[ind_up].first <= queries[ind_q].first.first){
            if (b[updates[ind_up].second]) update(dis[updates[ind_up].second], dis[updates[ind_up].second]); 
            ind_up++; 
        }
        res[queries[ind_q].first.second] = query(dis[queries[ind_q].second.first], fin[queries[ind_q].second.first]); 
        ind_q++; 
    }

    for (int v : res) cout << v << "\n"; 
}