#include <iostream>
#include <vector>
#include <set>
#include <algorithm>
#include <random>

using namespace std;

void solve();

int main() {
    ios::sync_with_stdio(0);
    cin.tie(0);
    cout.tie(0);
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#endif
    solve();
}

const int N = 2e5 + 100, INF = 1e18;

mt19937_64 rnd(331);

struct node {
    node* l = nullptr;
    node* r = nullptr;

    int x, sz;
    long long y;

    node(int x): x(x), y(rnd()), sz(1) {}
};

int size(node* t) {
    if (t == nullptr) return 0;
    return t->sz;
}

void upd(node* t) {
    t->sz = size(t->l) + size(t->r) + 1;
}

node* merge(node* a, node* b) {
    if (a == nullptr) return b;
    if (b == nullptr) return a;
    if (a->y > b->y) {
        a->r = merge(a->r, b);
        upd(a);
        return a;
    } else {
        b->l = merge(a, b->l);
        upd(b);
        return b;
    }
}

pair<node*, node*> split(node* a, int k) {
    if (a == nullptr) return {nullptr, nullptr};
    if (a->x <= k) {
        pair<node*, node*> z = split(a->r, k);
        a->r = z.first;
        upd(a);
        return {a, z.second};
    } else {
        pair<node*, node*> z = split(a->l, k);
        a->l = z.second;
        upd(a);
        return {z.first, a};
    }
}

struct segment_tree {
    node* t[4 * N];

//    segment_tree() {
//        for (int i = 0; i < 4 * N; i++) {
//            t[i] = -INF;
//        }
//    }

//    void build(int v, int l, int r, vector<int> &a) {
//        if (l == r) {
//            if (a[l] != -INF) {
//                t[v].push_back(a[l]);
//            }
//        } else {
//            int m = (r + l) / 2;
//            build(v * 2, l, m, a);
//            build(v * 2 + 1, m + 1, r, a);
//            for (int x : t[v * 2]) t[v].push_back(x);
//            for (int x : t[v * 2 + 1]) t[v].push_back(x);
//            sort(t[v].begin(), t[v].end());
//        }
//    }

//    int get(int v, int l, int r, int tl, int tr, int val) {
//        if (l >= tl && r <= tr) {
//            int ll = -1, rr = t[v].size();
//            while (ll + 1 < rr) {
//                int m = (rr + ll) / 2;
//                if (t[v][m] <= val) {
//                    ll = m;
//                } else {
//                    rr = m;
//                }
//            }
//            return ll + 1;
//        }
//        if (l > tr || r < tl) {
//            return 0;
//        }
//        int m = (r + l) / 2;
//        return get(v * 2, l, m, tl, tr, val) + get(v * 2 + 1, m + 1, r, tl, tr, val);
//    }

    int get(int v, int l, int r, int tl, int tr, int val) {
        if (l >= tl && r <= tr) {
            pair<node*, node*> z = split(t[v], val);
            int res = size(z.first);
            t[v] = merge(z.first, z.second);
            return res;
        }
        if (l > tr || r < tl) {
            return 0;
        }
        int m = (r + l) / 2;
        return get(v * 2, l, m, tl, tr, val) + get(v * 2 + 1, m + 1, r, tl, tr, val);
    }

    void add(int v, int l, int r, int pos, int val) {
        if (l == r) {
            t[v] = new node(val);
        } else {
            int m = (r + l) / 2;
            if (pos <= m) add(v * 2, l, m, pos, val);
            else add(v * 2 + 1, m + 1, r, pos, val);
            pair<node*, node*> z = split(t[v], val);
            t[v] = merge(z.first, merge(new node(val), z.second));
        }
    }
};

struct segment_tree2 {
    vector<int> t[4 * N];

//    segment_tree() {
//        for (int i = 0; i < 4 * N; i++) {
//            t[i] = -INF;
//        }
//    }

    void build(int v, int l, int r, vector<int> &a) {
        if (l == r) {
            if (a[l] != -INF) {
                t[v].push_back(a[l]);
            }
        } else {
            int m = (r + l) / 2;
            build(v * 2, l, m, a);
            build(v * 2 + 1, m + 1, r, a);
            for (int x : t[v * 2]) t[v].push_back(x);
            for (int x : t[v * 2 + 1]) t[v].push_back(x);
            sort(t[v].begin(), t[v].end());
        }
    }

    int get(int v, int l, int r, int tl, int tr, int val) {
        if (l >= tl && r <= tr) {
            int ll = -1, rr = t[v].size();
            while (ll + 1 < rr) {
                int m = (rr + ll) / 2;
                if (t[v][m] <= val) {
                    ll = m;
                } else {
                    rr = m;
                }
            }
            return ll + 1;
        }
        if (l > tr || r < tl) {
            return 0;
        }
        int m = (r + l) / 2;
        return get(v * 2, l, m, tl, tr, val) + get(v * 2 + 1, m + 1, r, tl, tr, val);
    }
};

int n, q, tin[N], tout[N], timer = -1, b[N], ans, pp[N], d[N];
vector<int> a[N];
segment_tree tree;

void dfs1(int v, int p) {
    pp[v] = p;
    tin[v] = ++timer;
    for (int to : a[v]) {
        if (to == p) continue;
        d[to] = d[v] + 1;
        dfs1(to, v);
    }
    tout[v] = timer;
}

void dfs2(int v, int p, int d) {
    b[v] = 1;
    if (d == 0) return;
    for (int to : a[v]) {
        if (to == p) continue;
        dfs2(to, v, d - 1);
    }
    tout[v] = timer;
}

void dfs3(int v, int p, int d) {
    ans += b[v];
    if (d == 0) return;
    for (int to : a[v]) {
        if (to == p) continue;
        dfs3(to, v, d - 1);
    }
}

void solve() {
    cin >> n;
    for (int i = 1; i < n; i++) {
        int f, s;
        cin >> f >> s;
        f--, s--;
        a[f].push_back(s);
        a[s].push_back(f);
    }
    dfs1(0, 0);
    vector<int> v(n);
    for (int i = 0; i < n; i++) {
        cin >> b[i];
        if (b[i]) {
            tree.add(1, 0, n - 1, tin[i], d[i]);
//            v[tin[i]] = d[i];
        } else {
            tree.add(1, 0, n - 1, tin[i], INF);
//            v[tin[i]] = -INF;
        }
    }
//    tree.build(1, 0, n - 1, v);
    cin >> q;
    for (int qq = 0; qq < q; qq++) {
        int t, x, k;
        cin >> t >> x >> k;
        x--;
        if (t == 1) {
        } else {
            cout << tree.get(1, 0, n - 1, tin[x], tout[x], d[x] + k) << '\n';
        }
    }
//    if (n <= 1e4 && q <= 1e4) {
//        for (int qq = 0; qq < q; qq++) {
//            int t, x, k;
//            cin >> t >> x >> k;
//            x--;
//            if (t == 1) {
//                dfs2(x, pp[x], k);
//            } else {
//                ans = 0;
//                dfs3(x, pp[x], k);
//                cout << ans << '\n';
//            }
//        }
//        return;
//    }
//    for (int qq = 0; qq < q; qq++) {
//        int t, x, k;
//        cin >> t >> x >> k;
//        x--;
//        if (t == 1) {
//            tree.update(1, 0, n - 1, tin[x], tout[x]);
//        } else {
//            cout << tree.get(1, 0, n - 1, tin[x], tout[x]) << '\n';
//        }
//    }
}