#include <iostream>
#include <vector>
#include <set>
#include <algorithm>

using namespace std;

void solve();

int main() {
    ios::sync_with_stdio(0);
    cin.tie(0);
    cout.tie(0);
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#endif
    solve();
}

const int N = 2e5 + 100, INF = 1e18;

struct segment_tree {
    vector<int> t[4 * N], fen[4 * N];

    int get(int v, int e) {
        int res = 0;
        for (; e >= 0; e = (e & (e + 1)) - 1) {
            res += fen[v][e];
        }
        return res;
    }

    void add(int v, int e) {
        for (; e < fen[v].size(); e |= e + 1) {
            fen[v][e]++;
        }
    }

    void build(int v, int l, int r, vector<int> &a) {
        if (l == r) {
            t[v].push_back(a[l]);
            fen[v].resize(1);
        } else {
            int m = (r + l) / 2;
            build(v * 2, l, m, a);
            build(v * 2 + 1, m + 1, r, a);
            t[v].resize(t[v * 2].size() + t[v * 2 + 1].size());
            fen[v].resize(t[v].size());
            merge(t[v * 2].begin(), t[v * 2].end(), t[v * 2 + 1].begin(), t[v * 2 + 1].end(), t[v].begin());
        }
    }

    int get(int v, int l, int r, int tl, int tr, int val) {
        if (l >= tl && r <= tr) {
            int ll = -1, rr = t[v].size();
            while (ll + 1 < rr) {
                int m = (rr + ll) / 2;
                if (t[v][m] <= val) {
                    ll = m;
                } else {
                    rr = m;
                }
            }
            if (ll == -1)
                return 0;
            return get(v, ll);
        }
        if (l > tr || r < tl) {
            return 0;
        }
        int m = (r + l) / 2;
        return get(v * 2, l, m, tl, tr, val) + get(v * 2 + 1, m + 1, r, tl, tr, val);
    }

    void add(int v, int l, int r, int pos, int val) {
        int ll = 0, rr = t[v].size();
        while (ll + 1 < rr) {
            int m = (rr + ll) / 2;
            if (t[v][m] <= val) {
                ll = m;
            } else {
                rr = m;
            }
        }
//        cout << l << " " << r << " " << ll << endl;
        add(v, ll);
        if (l == r) {
            return;
        }
        int m = (r + l) / 2;
        if (pos <= m) add(v * 2, l, m, pos, val);
        else add(v * 2 + 1, m + 1, r, pos, val);
    }
};

int n, q, tin[N], tout[N], timer = -1, b[N], d[N];
vector<int> a[N];
segment_tree tree;

void dfs1(int v, int p) {
    tin[v] = ++timer;
    for (int to : a[v]) {
        if (to == p) continue;
        d[to] = d[v] + 1;
        dfs1(to, v);
    }
    tout[v] = timer;
}

void solve() {
    cin >> n;
    for (int i = 1; i < n; i++) {
        int f, s;
        cin >> f >> s;
        f--, s--;
        a[f].push_back(s);
        a[s].push_back(f);
    }
    dfs1(0, 0);
    vector<int> v(n);
    for (int i = 0; i < n; i++) {
        v[tin[i]] = d[i];
    }
    tree.build(1, 0, n - 1, v);
    for (int i = 0; i < n; i++) {
        cin >> b[i];
        if (b[i]) {
            tree.add(1, 0, n - 1, tin[i], d[i]);
        }
    }
    cin >> q;
    for (int qq = 0; qq < q; qq++) {
        int t, x, k;
        cin >> t >> x >> k;
        x--;
        if (t == 1) {
        } else {
            cout << tree.get(1, 0, n - 1, tin[x], tout[x], d[x] + k) << '\n';
        }
    }
}