#include <iostream>
#include <vector>

using namespace std;

const int NMAX = 2e5 + 1;
const int INF = 1e9;

int N, In[NMAX], ROOT;
bool B[NMAX];
vector < int > G[NMAX];

int Q;

int M = 0, V[(NMAX << 1)];
int in[NMAX], out[NMAX];

struct NodeT
{
    int Min, ap;

    NodeT ()
    {
        Min = INF;
        ap = 0;
    }
};

class SegmentTree
{
    NodeT A[(NMAX << 3)];
    int Lazy[(NMAX << 3)];

private:
    inline void Go (int Node, int a, int b)
    {
        if(a == b)
            return;

        if(Lazy[Node] == 0)
            return;

        Lazy[(Node << 1)] += Lazy[Node], Lazy[((Node << 1) + 1)] += Lazy[Node];

        A[(Node << 1)].Min += Lazy[Node], A[((Node << 1) + 1)].Min += Lazy[Node];

        Lazy[Node] = 0;

        return;
    }

public:
    inline void Initialize (int Node, int a, int b)
    {
        if(a == b)
        {
            A[Node].Min = B[V[a]];
            A[Node].ap = 1;

            return;
        }

        int Mid = ((a + b) >> 1);

        Initialize((Node << 1), a, Mid), Initialize(((Node << 1) + 1), Mid + 1, b);

        if(A[(Node << 1)].Min < A[((Node << 1) + 1)].Min)
            A[Node] = A[(Node << 1)];
        else if(A[(Node << 1)].Min > A[((Node << 1) + 1)].Min)
            A[Node] = A[((Node << 1) + 1)];
        else
        {
            A[Node].Min = A[(Node << 1)].Min;
            A[Node].ap = A[(Node << 1)].ap + A[((Node << 1) + 1)].ap;
        }

        return;
    }

    inline void Update (int Node, int a, int b, int ua, int ub, int Val)
    {
        if(ua <= a && b <= ub)
        {
            A[Node].Min += Val;

            Lazy[Node] += Val;

            return;
        }

        Go(Node, a, b);

        int Mid = ((a + b) >> 1);

        if(ua <= Mid)
            Update((Node << 1), a, Mid, ua, ub, Val);

        if(ub > Mid)
            Update(((Node << 1) + 1), Mid + 1, b, ua, ub, Val);

        if(A[(Node << 1)].Min < A[((Node << 1) + 1)].Min)
            A[Node] = A[(Node << 1)];
        else if(A[(Node << 1)].Min > A[((Node << 1) + 1)].Min)
            A[Node] = A[((Node << 1) + 1)];
        else
        {
            A[Node].Min = A[(Node << 1)].Min;
            A[Node].ap = A[(Node << 1)].ap + A[((Node << 1) + 1)].ap;
        }

        return;
    }

    inline NodeT Query (int Node, int a, int b, int qa, int qb)
    {
        if(qa <= a && b <= qb)
            return A[Node];

        Go(Node, a, b);

        int Mid = ((a + b) >> 1);
        NodeT p_Left, p_Right;

        if(qa <= Mid)
            p_Left = Query((Node << 1), a, Mid, qa, qb);

        if(qb > Mid)
            p_Right = Query(((Node << 1) + 1), Mid + 1, b, qa, qb);

        if(p_Left.Min < p_Right.Min)
            return p_Left;
        else if(p_Left.Min > p_Right.Min)
            return p_Right;

        NodeT ans;
        ans.Min = p_Left.Min;
        ans.ap = p_Left.ap + p_Right.ap;

        return ans;
    }
} AINT;

static inline void Read ()
{
    ios_base :: sync_with_stdio(false);
    cin.tie(nullptr);

    cin >> N;

    for(int i = 1; i < N; ++i)
    {
        int x = 0, y = 0;
        cin >> x >> y;

        ++In[y];

        G[x].push_back(y);
    }

    for(int i = 1; i <= N; ++i)
        cin >> B[i];

    return;
}

static inline void Find_Root ()
{
    for(int i = 1; i <= N && !ROOT; ++i)
        if(In[i] == 0)
            ROOT = i;

    return;
}

static inline void DFS (int Node)
{
    V[++M] = Node;
    in[Node] = M;

    for(auto it : G[Node])
        DFS(it);

    V[++M] = Node;
    out[Node] = M;

    return;
}

static inline void Precalculation ()
{
    Find_Root();

    DFS(ROOT);

    AINT.Initialize(1, 1, (N << 1));

    return;
}

static inline void TestCase ()
{
    int type = 0, x = 0, k = 0;
    cin >> type >> x >> k;

    if(type == 1)
    {
        AINT.Update(1, 1, (N << 1), in[x], out[x], +1);

        return;
    }

    NodeT now = AINT.Query(1, 1, (N << 1), in[x], out[x]);

    if(now.Min == 0)
        cout << (((out[x] - in[x] + 1) - now.ap) >> 1) << '\n';
    else
        cout << ((out[x] - in[x] + 1) >> 1) << '\n';

    return;
}

static inline void Solve ()
{
    Precalculation();

    cin >> Q;
    while(Q--)
        TestCase();

    return;
}

int main()
{
    Read();

    Solve();

    return 0;
}
