#pragma GCC optimize("Ofast,unroll-loops")
#pragma GCC target("avx2,bmi,bmi2,lzcnt,popcnt")

#include <iostream>
#include <vector>

using namespace std;

using ll = long long;

void dfs(int v, int& timer, vector<vector<int>>& adj, vector<int>& start, vector<int>& end)
{
    start[v] = timer++;
    for (int u : adj[v]) dfs(u, timer, adj, start, end);
    end[v] = timer - 1;
}

struct SegmentTree
{
    int n;
    vector<int> tree;
    vector<int> mod;

    void build(int v, int l, int r, vector<int> &a)
    {
        if (l > r) return;
        if (l == r)
        {
            tree[v] = a[l];
            return;
        }
        int m = (l + r) >> 1;
        build(v * 2, l, m, a);
        build(v * 2 + 1, m + 1, r, a);
        tree[v] = tree[v * 2] + tree[v * 2 + 1];
    }

    SegmentTree(vector<int> &a)
    {
        n = a.size();
        tree.resize(4 * n + 5);
        mod.resize(4 * n + 5);
        build(1, 0, n - 1, a);
    }

    void push(int v, int l, int r)
    {
        if (mod[v] != 0)
        {
            int m = (l + r) >> 1;
            mod[v * 2] = mod[v];
            mod[v * 2 + 1] = mod[v];
            tree[v * 2] = (m - l + 1) * mod[v];
            tree[v * 2 + 1] = (r - m) * mod[v];
            mod[v] = 0;
        }
    }

    void update(int a, int b, int val, int v = 1, int l = 0, int r = -1)
    {
        if (r == -1) r = n - 1;
        if (a > r || b < l) return;
        if (l == a && r == b)
        {
            tree[v] = val * (r - l + 1);
            mod[v] = val;
            return;
        }
        int m = (l + r) >> 1;
        push(v, l, r);
        update(a, min(b, m), val, v * 2, l, m);
        update(max(a, m + 1), b, val, v * 2 + 1, m + 1, r);
        tree[v] = tree[v * 2] + tree[v * 2 + 1];
    }

    int get(int a, int b, int v = 1, int l = 0, int r = -1)
    {
        if (r == -1) r = n - 1;
        if (a > r || b < l) return 0;
        if (l == a && r == b) return tree[v];
        int m = (l + r) >> 1;
        push(v, l, r);
        return get(a, min(b, m), v * 2, l, m) + get(max(a, m + 1), b, v * 2 + 1, m + 1, r);
    }

};

signed main()
{
    cin.tie(0)->sync_with_stdio(0);
    int n;
    cin >> n;
    vector<vector<int>> adj(n + 1);
    for (int i = 0; i < n - 1; ++i)
    {
        int x, y;
        cin >> x >> y;
        adj[x].push_back(y);
    }
    vector<int> b(n + 1);
    for (int i = 1; i <= n; ++i) cin >> b[i];
    vector<int> tob(n);
    vector<int> start(n + 1), end(n + 1);
    int timer = 0;
    dfs(1, timer, adj, start, end);
    for (int i = 1; i <= n; ++i) tob[start[i]] = b[i];
    SegmentTree st(tob);
    int q;
    cin >> q;
    while (q--)
    {
        int t, x, k;
        cin >> t >> x >> k;
        if (t == 2) cout << st.get(start[x], end[x]) << '\n';
        else st.update(start[x], end[x], 1);
    }
    return 0;
}/*
10
1 2
1 3
3 4
3 5
3 6
4 7
4 8
8 9
8 10
0 1 0 1 0 1 0 1
0 1
9
2 1 1
2 4 4
2 3 0
1 1 2
2 3 4
1 4 1
2 1 1
2 4 4
2 3 2

 */
