#include <set>
#include <vector>
#include <iomanip>
#include <iostream>
#include <algorithm>

using namespace std;

const int MAXN = 200'200;
const int K = 18;
const int KK = 448;

int lg[MAXN];

vector<int> G[MAXN];
int up[MAXN][K];
int depth[MAXN];
int mx_depth[MAXN];
vector<int> levels[MAXN];
vector<int> paths[MAXN];
int path_id[MAXN];
int path_ind[MAXN];
int level_id[MAXN];
vector<int> order;

int all_sum[MAXN];
vector<int> pref_all_sums[MAXN];

void dfs1(int v, int w) {
  order.emplace_back(v);
  depth[v] = w;
  mx_depth[v] = w;
  level_id[v] = levels[w].size();
  levels[w].emplace_back(v);
  all_sum[v] = 1;
  if (G[v].empty()) {
    path_id[v] = v;
    path_ind[v] = 0;
    paths[v].emplace_back(v);
    return;
  }
  int best = -1;
  for (int u : G[v]) {
    up[u][0] = v;
    dfs1(u, w + 1);
    all_sum[v] += all_sum[u];
    mx_depth[v] = max(mx_depth[v], mx_depth[u]);
    if (best == -1 || mx_depth[u] > mx_depth[best])
      best = u;
  }
  path_id[v] = path_id[best];
  path_ind[v] = paths[path_id[v]].size();
  paths[path_id[v]].emplace_back(v);
}

int la(int v, int d) {
  if (d == 0)
    return v;
  int k = lg[d];
  v = up[v][k];
  d -= 1 << k;
  return paths[path_id[v]][path_ind[v] + d];
}

bool good[MAXN];
int sum[MAXN];
vector<int> pref_sums[MAXN];
set<pair<int, int>> to_switch;
set<int> alive_levels[MAXN];

void recalc(int n) {
  for (auto [w, v] : to_switch)
    good[v] = false;
  to_switch.clear();
  for (int v : order) {
    sum[v] = good[v];
    for (int u : G[v])
      sum[v] += sum[u];
  }
  for (int i = 0; i < n; ++i) {
    pref_sums[i].resize(levels[i].size() + 1);
    for (int j = 0; j < (int) levels[i].size(); ++j)
      pref_sums[i][j + 1] = pref_sums[i][j] + sum[levels[i][j]];
  }
}

int get(int v, int k, int n, bool kek = true) {
  int ans = sum[v];
  int w = depth[v] + k + 1;
  if (w < n) {
    int left = -1, right = levels[w].size();
    while (right - left > 1) {
      int mid = (left + right) / 2;
      if (level_id[la(levels[w][mid], k + 1)] >= level_id[v])
        right = mid;
      else
        left = mid;
    }
    ans += pref_sums[w][right];
    left = -1, right = levels[w].size();
    while (right - left > 1) {
      int mid = (left + right) / 2;
      if (level_id[la(levels[w][mid], k + 1)] > level_id[v])
        right = mid;
      else
        left = mid;
    }
    ans -= pref_sums[w][right];
  }
  if (kek) {
    for (auto [w, u]: to_switch) {
      if (w >= depth[v] && w <= depth[v] + k && la(u, w - depth[v]) == v)
        --ans;
    }
  }
  return ans;
}


int get_all(int v, int k, int n) {
  int ans = all_sum[v];
  int w = depth[v] + k + 1;
  if (w < n) {
    int left = -1, right = levels[w].size();
    while (right - left > 1) {
      int mid = (left + right) / 2;
      if (level_id[la(levels[w][mid], k + 1)] >= level_id[v])
        right = mid;
      else
        left = mid;
    }
    ans += pref_all_sums[w][right];
    left = -1, right = levels[w].size();
    while (right - left > 1) {
      int mid = (left + right) / 2;
      if (level_id[la(levels[w][mid], k + 1)] > level_id[v])
        right = mid;
      else
        left = mid;
    }
    ans -= pref_all_sums[w][right];
  }
  return ans;
}

vector<int> solve(int n, vector<pair<int, int>> edges, vector<int> start, vector<pair<int, pair<int, int>>> queries) {
  for (auto [x, y] : edges)
    G[x].emplace_back(y);

  for (int i = 0; i < n; ++i)
    good[i] = !start[i];

  order.reserve(n);
  dfs1(0, 0);
  for (int i = 0; i < n; ++i) {
    if (good[i])
      alive_levels[depth[i]].emplace(level_id[i]);
  }
  for (int j = 1; j < K; ++j) {
    for (int i = 0; i < n; ++i)
      up[i][j] = up[up[i][j - 1]][j - 1];
  }
  for (int i = 0; i < n; ++i) {
    int k = paths[i].size();
    while (k--) {
      int v = paths[i].back();
      if (up[v][0] != v)
        paths[i].emplace_back(up[v][0]);
      else
        break;
    }
  }

  for (int i = 0; i < n; ++i) {
    pref_all_sums[i].resize(levels[i].size() + 1);
    for (int j = 0; j < (int) levels[i].size(); ++j)
      pref_all_sums[i][j + 1] = pref_all_sums[i][j] + all_sum[levels[i][j]];
  }

  vector<int> checked(n, -1);
  reverse(order.begin(), order.end());
  recalc(n);

  vector<int> ans;
  for (auto tmp : queries) {
    int t = tmp.first, v = tmp.second.first, k = tmp.second.second;
    if (t == 1) {
      while (checked[v] < k) {
        int left = checked[v], right = n;
        vector<int> interesting;
        for (auto [w, u] : to_switch) {
          if (w >= depth[v] && la(u, w - depth[v]) == v)
            interesting.emplace_back(w);
        }
        while (right - left > 1) {
          int mid = (left + right) / 2;
          int tmp = get(v, mid, n, false) - (upper_bound(interesting.begin(), interesting.end(), depth[v] + mid) - interesting.begin());
          if (tmp == 0)
            left = mid;
          else
            right = mid;
        }
        checked[v] = left;
        int w = depth[v] + right;
        if (w >= n || w > depth[v] + k)
          break;
        left = -1, right = levels[w].size();
        while (right - left > 1) {
          int mid = (left + right) / 2;
          if (level_id[la(levels[w][mid], w - depth[v])] >= level_id[v])
            right = mid;
          else
            left = mid;
        }
        int ll = right;
        left = -1, right = levels[w].size();
        while (right - left > 1) {
          int mid = (left + right) / 2;
          if (level_id[la(levels[w][mid], w - depth[v])] > level_id[v])
            right = mid;
          else
            left = mid;
        }
        int rr = right;
        while (true) {
          auto tmp = alive_levels[w].lower_bound(ll);
          if (tmp != alive_levels[w].end() && (*tmp) < rr) {
            to_switch.emplace(w, levels[w][*tmp]);
            alive_levels[w].erase(tmp);
          } else {
            break;
          }
        }
        if (to_switch.size() > KK)
          recalc(n);
        ++checked[v];
      }
    } else {
      ans.emplace_back(get_all(v, k, n) - get(v, k, n));
    }
  }
  order.clear();
  to_switch.clear();
  for (int i = 0; i < n; ++i) {
    levels[i].clear();
    alive_levels[i].clear();
    paths[i].clear();
    G[i].clear();
    pref_sums[i].clear();
    pref_all_sums[i].clear();
  }
  return ans;
}

void dumb_fs1(int v, int k, vector<int> &state) {
  state[v] = 1;
  if (k == 0)
    return;
  for (int u : G[v])
    dumb_fs1(u, k - 1, state);
}

int dumb_fs2(int v, int k, vector<int> &state) {
  int ans = state[v];
  if (k == 0)
    return ans;
  for (int u : G[v])
    ans += dumb_fs2(u, k - 1, state);
  return ans;
}

vector<int> dumb(int n, vector<pair<int, int>> edges, vector<int> state, vector<pair<int, pair<int, int>>> queries) {
  for (auto [x, y] : edges)
    G[x].emplace_back(y);
  vector<int> ans;
  for (auto tmp : queries) {
    int t = tmp.first, v = tmp.second.first, k = tmp.second.second;
    if (t == 1) {
      dumb_fs1(v, k, state);
    } else {
      ans.emplace_back(dumb_fs2(v, k, state));
    }
  }
  for (int i = 0; i < n; ++i)
    G[i].clear();
  return ans;
}

//#define STRESS

int main() {
#ifdef DEBUG
  freopen("input.txt", "r", stdin);
  freopen("output.txt", "w", stdout);
#else
  ios::sync_with_stdio(false);
  cin.tie(0);
  cout.tie(0);
#endif

  lg[0] = -1;
  for (int i = 1; i < MAXN; ++i)
    lg[i] = lg[i / 2] + 1;

#ifdef STRESS
  srand(228);
  for (int _ = 0; _ < 1'000'000; ++_) {
    int n = rand() % 20 + 1;
    vector<pair<int, int>> edges(n - 1);
    for (int i = 1; i < n; ++i)
      edges[i - 1] = make_pair(rand() % i, i);
    vector<int> start(n);
    for (int i = 0; i < n; ++i) {
      if (rand() % 4)
        start[i] = 0;
      else
        start[i] = 1;
    }
    int q = 20;
    vector<pair<int, pair<int, int>>> queries(q);
    for (int i = 0; i < q; ++i) {
      queries[i].first = 1 + rand() % 2;
      queries[i].second.first = rand() % n;
      queries[i].second.second = rand() % n;
    }
    auto ans1 = solve(n, edges, start, queries);
    auto ans2 = dumb(n, edges, start, queries);
    if (ans1 != ans2) {
      cerr << "BUG!!!\n";
      cerr << n << '\n';
      for (int i = 0; i < n - 1; ++i)
        cerr << edges[i].first + 1 << ' ' << edges[i].second + 1 << '\n';
      for (int x : start)
        cerr << x << ' ';
      cerr << '\n';
      cerr << q << '\n';
      for (auto tmp : queries)
        cerr << tmp.first << ' ' << tmp.second.first + 1 << ' ' << tmp.second.second << '\n';
      cerr << "wrong: ";
      for (int x : ans1)
        cerr << x << ' ';
      cerr << "\nright: ";
      for (int x : ans2)
        cerr << x << ' ';
      cerr << "\n\n\n";
    }
  }
#else
  int n;
  cin >> n;
  vector<pair<int, int>> edges(n - 1);
  for (int i = 0; i < n - 1; ++i) {
    cin >> edges[i].first >> edges[i].second;
    --edges[i].first;
    --edges[i].second;
  }
  vector<int> start(n);
  for (int i = 0; i < n; ++i)
    cin >> start[i];
  int q;
  cin >> q;
  vector<pair<int, pair<int, int>>> queries(q);
  for (int i = 0; i < q; ++i) {
    cin >> queries[i].first >> queries[i].second.first >> queries[i].second.second;
    --queries[i].second.first;
    queries[i].second.second = min(queries[i].second.second, n - 1);
  }
  auto ans = solve(n, edges, start, queries);
  for (int x : ans)
    cout << x << '\n';
#endif
  return 0;
}