#include <set>
#include <vector>
#include <iomanip>
#include <iostream>
#include <algorithm>

using namespace std;

const int MAXN = 200'200;

int fenw[MAXN];

void add(int i, int x) {
  for (; i < MAXN; i = (i | (i + 1)))
    fenw[i] += x;
}

int get(int i) {
  int ans = 0;
  for (; i >= 0; i = (i & (i + 1)) - 1)
    ans += fenw[i];
  return ans;
}

vector<int> G[MAXN];
int tin[MAXN];
int tout[MAXN];
vector<int> order;

int sz[MAXN];

void dfs1(int v, int w) {
  tin[v] = order.size();
  order.emplace_back(v);
  sz[v] = 1;
  for (int i = 0; i < (int) G[v].size(); ++i) {
    dfs1(G[v][i], w + 1);
    sz[v] += sz[G[v][i]];
  }
  tout[v] = order.size();
}

bool good[MAXN];
bool kek[MAXN];

void go_dumb(int v) {
  if (kek[v])
    return;
  kek[v] = true;
  if (good[v])
    add(tin[v], -1);
  for (int u : G[v])
    go_dumb(u);
}

vector<int> solve(int n, vector<pair<int, int>> edges, vector<int> start, vector<pair<int, pair<int, int>>> queries) {
  for (auto [x, y] : edges)
    G[x].emplace_back(y);

  for (int i = 0; i < n; ++i)
    good[i] = !start[i];

  order.reserve(n);
  dfs1(0, 0);

  fill(fenw, fenw + n, 0);
  for (int i = 0; i < n; ++i)
    add(tin[i], good[i]);

  vector<int> ans;
  for (auto tmp : queries) {
    int t = tmp.first, v = tmp.second.first, k = tmp.second.second;
    if (t == 1) {
      go_dumb(v);
    } else {
      ans.emplace_back(sz[v] - get(tout[v] - 1) + get(tin[v] - 1));
    }
  }
  order.clear();
  for (int i = 0; i < n; ++i)
    G[i].clear();
  return ans;
}

void dumb_fs1(int v, int k, vector<int> &state) {
  state[v] = 1;
  if (k == 0)
    return;
  for (int u : G[v])
    dumb_fs1(u, k - 1, state);
}

int dumb_fs2(int v, int k, vector<int> &state) {
  int ans = state[v];
  if (k == 0)
    return ans;
  for (int u : G[v])
    ans += dumb_fs2(u, k - 1, state);
  return ans;
}

vector<int> dumb(int n, vector<pair<int, int>> edges, vector<int> state, vector<pair<int, pair<int, int>>> queries) {
  for (auto [x, y] : edges)
    G[x].emplace_back(y);
  vector<int> ans;
  for (auto tmp : queries) {
    int t = tmp.first, v = tmp.second.first, k = tmp.second.second;
    if (t == 1) {
      dumb_fs1(v, k, state);
    } else {
      ans.emplace_back(dumb_fs2(v, k, state));
    }
  }
  for (int i = 0; i < n; ++i)
    G[i].clear();
  return ans;
}

//#define STRESS

int main() {
#ifdef DEBUG
  freopen("input.txt", "r", stdin);
  freopen("output.txt", "w", stdout);
#else
  ios::sync_with_stdio(false);
  cin.tie(0);
  cout.tie(0);
#endif

#ifdef STRESS
  srand(228);
  for (int _ = 0; _ < 1'000'000; ++_) {
    int n = rand() % 5 + 1;
    vector<pair<int, int>> edges(n - 1);
    for (int i = 1; i < n; ++i)
      edges[i - 1] = make_pair(rand() % i, i);
    vector<int> start(n);
    for (int i = 0; i < n; ++i) {
      if (rand() % 4)
        start[i] = 0;
      else
        start[i] = 1;
    }
    int q = 5;
    vector<pair<int, pair<int, int>>> queries(q);
    for (int i = 0; i < q; ++i) {
      queries[i].first = 1 + rand() % 2;
      queries[i].second.first = rand() % n;
      queries[i].second.second = n;
    }
    auto ans1 = solve(n, edges, start, queries);
    auto ans2 = dumb(n, edges, start, queries);
    if (ans1 != ans2) {
      cerr << "BUG!!!\n";
      cerr << n << '\n';
      for (int i = 0; i < n - 1; ++i)
        cerr << edges[i].first + 1 << ' ' << edges[i].second + 1 << '\n';
      for (int x : start)
        cerr << x << ' ';
      cerr << '\n';
      cerr << q << '\n';
      for (auto tmp : queries)
        cerr << tmp.first << ' ' << tmp.second.first + 1 << ' ' << tmp.second.second << '\n';
      cerr << "wrong: ";
      for (int x : ans1)
        cerr << x << ' ';
      cerr << "\nright: ";
      for (int x : ans2)
        cerr << x << ' ';
      cerr << "\n\n\n";
    }
  }
#else
  int n;
  cin >> n;
  vector<pair<int, int>> edges(n - 1);
  for (int i = 0; i < n - 1; ++i) {
    cin >> edges[i].first >> edges[i].second;
    --edges[i].first;
    --edges[i].second;
  }
  vector<int> start(n);
  for (int i = 0; i < n; ++i)
    cin >> start[i];
  int q;
  cin >> q;
  vector<pair<int, pair<int, int>>> queries(q);
  for (int i = 0; i < q; ++i) {
    cin >> queries[i].first >> queries[i].second.first >> queries[i].second.second;
    --queries[i].second.first;
    queries[i].second.second = min(queries[i].second.second, n - 1);
  }
  auto ans = solve(n, edges, start, queries);
  for (int x : ans)
    cout << x << '\n';
#endif
  return 0;
}