#include <bits/stdc++.h>

using namespace std;


int n;

vector<int> time_first_learned;
vector<vector<pair<int, int>>> updates;
vector<vector<int>> adj;


class SegmentTreeMin {
	vector<int> a;
	int n;

public:
	SegmentTreeMin() {
	}

	void resize(int n_) {
		n = n_;
		a.resize(2 * n, 1000000000);
	}

	int size() const {
		return n;
	}

	int get(int l, int r) const {
		l += n;
		r += n;
		int ans = 1000000000;
		while(l < r) {
			ans = min(ans, a[l]);
			ans = min(ans, a[r - 1]);
			l = (l + 1) / 2;
			r /= 2;
		}
		return ans;
	}

	int get(int i) const {
		return a[n + i];
	}

	void set(int i, int x) {
		i += n;
		a[i] = x;
		i /= 2;
		while(i >= 1) {
			a[i] = min(a[i * 2], a[i * 2 + 1]);
			i /= 2;
		}
	}
};


SegmentTreeMin time_first_learned_by_height;


void dfs1(int u, int h) {
	vector<pair<int, int>> restorer;
	for(auto [k, q]: updates[u]) {
		int hh = min(h + k, n);
		int oldval = time_first_learned_by_height.get(hh);
		restorer.emplace_back(hh, oldval);
		time_first_learned_by_height.set(hh, min(oldval, q));
	}
	time_first_learned[u] = min(time_first_learned[u], time_first_learned_by_height.get(h, time_first_learned_by_height.size()));
	for(int v: adj[u]) {
		dfs1(v, h + 1);
	}
	for(int j = (int)restorer.size() - 1; j >= 0; j--) {
		auto [i, x] = restorer[j];
		time_first_learned_by_height.set(i, x);
	}
}


vector<vector<pair<int, int>>> queries;
vector<int> answers;


class MergeSortTree {
	vector<vector<int>> a;
	int n;

	void build(int v, int vl, int vr, const vector<vector<int>>& data) {
		if(vr - vl == 1) {
			a[v] = data[vl];
			sort(a[v].begin(), a[v].end());
		} else {
			int vm = (vl + vr) / 2;
			build(v * 2, vl, vm, data);
			build(v * 2 + 1, vm, vr, data);
			a[v].resize(a[v * 2].size() + a[v * 2 + 1].size());
			merge(a[v * 2].begin(), a[v * 2].end(), a[v * 2 + 1].begin(), a[v * 2 + 1].end(), a[v].begin());
		}
	}

public:
	MergeSortTree() {
	}

	void build(const vector<vector<int>>& data) {
		n = data.size();
		a.resize(4 * n);
		build(1, 0, n, data);
	}

	int size() const {
		return n;
	}

	int get(int v, int vl, int vr, int l, int r, int low, int high) const {
		if(l == vl && r == vr) {
			return lower_bound(a[v].begin(), a[v].end(), high) - lower_bound(a[v].begin(), a[v].end(), low);
		}
		int vm = (vl + vr) / 2;
		if(r <= vm) {
			return get(v * 2, vl, vm, l, r, low, high);
		} else if(l >= vm) {
			return get(v * 2 + 1, vm, vr, l, r, low, high);
		} else {
			return get(v * 2, vl, vm, l, vm, low, high) + get(v * 2 + 1, vm, vr, vm, r, low, high);
		}
	}
	int get(int l, int r, int low, int high) const {
		return get(1, 0, n, l, r, low, high);
	}
};


class CompressedMergeSortTree {
	MergeSortTree st;
	vector<int> xx;

public:
	CompressedMergeSortTree() {
	}

	void build(const vector<pair<int, int>>& data) {
		for(auto [x, y]: data) {
			xx.push_back(x);
		}
		sort(xx.begin(), xx.end());
		xx.erase(unique(xx.begin(), xx.end()), xx.end());
		vector<vector<int>> y_by_x(xx.size());
		for(auto [x, y]: data) {
			y_by_x[lower_bound(xx.begin(), xx.end(), x) - xx.begin()].push_back(y);
		}
		st.build(y_by_x);
	}

	int get(int l, int r, int low, int high) const {
		return st.get(
			lower_bound(xx.begin(), xx.end(), l) - xx.begin(),
			lower_bound(xx.begin(), xx.end(), r) - xx.begin(),
			low,
			high
		);
	}
};



void dfs2(int u, int h, vector<pair<int, int>>& children) {
	for(int v: adj[u]) {
		vector<pair<int, int>> tmp_children;
		dfs2(v, h + 1, tmp_children);
		for(auto [hh, qq]: tmp_children) {
			children.emplace_back(hh, qq);
		}
	}
	children.emplace_back(h, time_first_learned[u]);

	CompressedMergeSortTree children_st;
	children_st.build(children);
	for(auto [k, q]: queries[u]) {
		answers[q] = children_st.get(0, h + k + 1, 0, q + 1);
	}
}


signed main() {
	ios_base::sync_with_stdio(0);
	cin.tie(0);


	cin >> n;

	adj.resize(n);
	for(int i = 0; i < n - 1; i++) {
		int u, v;
		cin >> u >> v;
		u--; v--;
		adj[u].push_back(v);
	}

	time_first_learned.resize(n);
	for(int i = 0; i < n; i++) {
		int b;
		cin >> b;
		time_first_learned[i] = b == 1 ? 0 : 1000000000;
	}


	int q;
	cin >> q;

	updates.resize(n);
	queries.resize(n);
	for(int i = 0; i < q; i++) {
		int c, x, k;
		cin >> c >> x >> k;
		x--;
		if(c == 1) {
			updates[x].emplace_back(k, i);
		} else if(c == 2) {
			queries[x].emplace_back(k, i);
		}
	}

	time_first_learned_by_height.resize(n + 1);
	dfs1(0, 0);

	vector<pair<int, int>> tmp;
	answers.resize(q, -1);
	dfs2(0, 0, tmp);

	for(int x: answers) {
		if(x != -1) {
			cout << x << "\n";
		}
	}

	return 0;
}
