#include<iostream>
#include<vector>
#include<cmath>
using namespace std;
struct segNode
{
    int l, r,  x;
};
const int MAXN=2e5, MAXM=(1<<int(ceil(log2(MAXN))));
vector<int> graph[MAXN];
int dgr[MAXN], pos[MAXN], sz[MAXN], b[MAXN];
vector<int> order;
segNode seg[2*MAXM];
bool lazy[2*MAXM];
void GetTreeOrder(int idx)
{
    pos[idx]=order.size(), sz[idx]=1;
    order.push_back(idx);
    for(int node:graph[idx])
    {
        GetTreeOrder(node);
        sz[idx]+=sz[node];
    }
    return;
}
void BuildSegTree(int n)
{
    int m=(1<<int(ceil(log2(n))));
    for(int i=m; i<m+n; i++)
    {
        seg[i]={i-m, i-m, b[i-m]};
        lazy[i]=false;
    }
    for(int i=m+n; i<2*m; i++)
    {
        seg[i]={i-m, i-m, 0};
        lazy[i]=false;
    }
    for(int i=m-1; i>0; i--)
    {
        seg[i]={seg[2*i].l, seg[2*i+1].r, seg[2*i].x+seg[2*i+1].x};
        lazy[i]=false;
    }
    return;
}
void Update(int idx, int l, int r)
{
    if(lazy[idx])
    {
        return;
    }
    if(l==seg[idx].l && r==seg[idx].r)
    {
        lazy[idx]=true;
        seg[idx].x=r-l+1;
        return;
    }
    if(l>seg[2*idx].r)
    {
        Update(2*idx+1, l, r);
        return;
    }
    if(r<seg[2*idx+1].l)
    {
        Update(2*idx, l, r);
        return;
    }
    Update(2*idx, l, seg[2*idx].r);
    Update(2*idx+1, seg[2*idx+1].l, r);
    return;
}
int GetCount(int idx, int l, int r)
{
    if(l==seg[idx].l && r==seg[idx].r)
    {
        return seg[idx].x;
    }
    if(lazy[idx])
    {
        lazy[2*idx]=true, lazy[2*idx+1]=true;
        seg[2*idx].x=seg[2*idx].r-seg[2*idx].l+1;
        seg[2*idx+1].x=seg[2*idx+1].r-seg[2*idx+1].l+1;
    }
    if(l>seg[2*idx].r)
    {
        return GetCount(2*idx+1, l, r);
    }
    if(r<seg[2*idx+1].l)
    {
        return GetCount(2*idx, l, r);
    }
    return GetCount(2*idx, l, seg[2*idx].r)+GetCount(2*idx+1, seg[2*idx+1].l, r);
}
int main()
{
    ios_base::sync_with_stdio(0);
    cin.tie(0);
    int n;
    cin >> n;
    for(int i=0; i<n; i++)
    {
        dgr[i]=0;
    }
    for(int i=0; i<n-1; i++)
    {
        int x, y;
        cin >> x >> y;
        x--, y--;
        graph[x].push_back(y);
        dgr[y]++;
    }
    for(int i=0; i<n; i++)
    {
        cin >> b[i];
    }
    for(int i=0; i<n; i++)
    {
        if(dgr[i]==0)
        {
            GetTreeOrder(i);
        }
    }
    BuildSegTree(n);
    int q;
    cin >> q;
    while(q--)
    {
        int t, x, k;
        cin >> t >> x >> k;
        x--;
        if(t==1)
        {
            Update(1, pos[x], pos[x]+sz[x]-1);
        }
        else
        {
            cout << GetCount(1, pos[x], pos[x]+sz[x]-1) << "\n";
        }
    }
    return 0;
}
/*
10
1 2
1 3
3 4
3 5
3 6
4 7
4 8
8 9
8 10
0 1 0 1 0 1 0 1
0 1
9
2 1 10
2 4 10
2 3 10
1 1 2
2 3 4
1 4 1
2 1 1
2 4 4
2 3 2

*/
