#include <bits/stdc++.h>

#pragma GCC optimize "-O3"

#define endl '\n'
#define trace(x) cerr << #x << " = " << x << endl

using namespace std;

const int MAX_LOG = 20;

void setIO() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
}

int n;
vector<vector<int>> children;
vector<int> p;
vector<int> knows;

void read() {
    cin >> n;

    children.resize(n + 1);
    p.resize(n + 1, -1);
    for (int i = 0; i < n - 1; i++) {
        int x, y;
        cin >> x >> y;

        children[x].emplace_back(y);
        p[y] = x;
    }

    knows.resize(n + 1);
    for (int i = 1; i <= n; i++) {
        cin >> knows[i];
    }
}

vector<vector<int>> levels;
vector<int> nodeLvl;

void bfsLevels() {
    queue<pair<int, int>> q;
    nodeLvl.resize(n + 1);

    q.emplace(1, 0);
    while (!q.empty()) {
        pair<int, int> now = q.front();
        q.pop();

        if (now.second == levels.size()) {
            levels.emplace_back();
        }

        levels.back().emplace_back(now.first);
        nodeLvl[now.first] = now.second;

        for (int child : children[now.first]) {
            q.emplace(child, now.second + 1);
        }
    }
}

vector<vector<int>> par;

void computePar() {
    par.resize(n + 1, vector<int>(MAX_LOG));

    for (int j = 1; j <= n; j++) {
        par[j][0] = p[j];
    }

    for (int i = 1; i < MAX_LOG; i++) {
        for (int j = 1; j <= n; j++) {
            if (par[j][i - 1] == -1) {
                par[j][i] = -1;
                continue;
            }

            par[j][i] = par[par[j][i - 1]][i - 1];
        }
    }
}

int getPar(int root, int lvl) {
    int res = root;
    for (int i = MAX_LOG - 1; i >= 0; i--) {
        if ((lvl >> i) & 1) {
            res = par[res][i];
        }

        if (res == -1) {
            break;
        }
    }

    return res;
}

vector<int> in, to;
int dfsTime = 1;

void dfsTree(int root) {
    in[root] = to[root] = dfsTime++;

    for (int child : children[root]) {
        dfsTree(child);
        to[root] = max(to[root], to[child]);
    }
}

vector<int> tree;

void update(int pos) {
    for (int i = pos; i < (int)tree.size(); i += i & -i) {
        tree[i]++;
    }
}

int query(int pos) {
    int answer = 0;
    for (int i = pos; i > 0; i -= i & -i) {
        answer += tree[i];
    }

    return answer;
}

int query(int l, int r) {
    int answer = query(r);
    if (l > 1) {
        answer -= query(l - 1);
    }

    return answer;
}

int solve(int x, int k) {
    int currLvl = nodeLvl[x];
    int toLvl = currLvl + k;
    int answer = query(in[x], to[x]);
    if (toLvl < levels.size()) {
        for (int node : levels[toLvl]) {
            if (getPar(node, k) == x) {
                answer -= query(in[node], to[node]);
                answer += knows[node];
            }
        }
    }

    return answer;
}

void solve() {
    bfsLevels();

    computePar();

    in.resize(n + 1);
    to.resize(n + 1);
    dfsTree(1);

    tree.resize(n + 1, 0);
    for (int i = 1; i <= n; i++) {
        if (knows[i]) {
            update(in[i]);
        }
    }

    int q;
    cin >> q;

    while (q--) {
        int type, x, k;
        cin >> type >> x >> k;

        cout << solve(x, k) << endl;
    }
}

int main() {
    setIO();

    read();

    solve();
}

/*
10
1 2
1 3
3 4
3 5
3 6
4 7
4 8
8 9
8 10
0 1 0 1 0 1 0 1 0 1
7
2 1 1
2 4 4
2 3 0
2 3 4
2 1 1
2 4 4
2 3 2
*/
